@@ -772,6 +772,17 @@ ULONG WINAPI RegisterTraceGuidsW( WMIDPREQUEST RequestAddress,
     FIXME("(%p, %p, %s, %u, %p, %s, %s, %p): stub\n", RequestAddress, RequestContext,
           debugstr_guid(ControlGuid), GuidCount, TraceGuidReg, debugstr_w(MofImagePath),
           debugstr_w(MofResourceName), RegistrationHandle);
+
+    if (TraceGuidReg)
+    {
+        ULONG i;
+        for (i = 0; i < GuidCount; i++)
+        {
+            FIXME("  register trace class %s\n", debugstr_guid(TraceGuidReg[i].Guid));
+            TraceGuidReg[i].RegHandle = (HANDLE)0xdeadbeef;
+        }
+    }
+    *RegistrationHandle = (TRACEHANDLE)0xdeadbeef;
     return ERROR_SUCCESS;
 }
 
@@ -907,6 +918,16 @@ BOOLEAN WINAPI EventProviderEnabled( REGHANDLE handle, UCHAR level, ULONGLONG ke
     return FALSE;
 }
 
+/******************************************************************************
+ * EventActivityIdControl [ADVAPI32.@]
+ *
+ */
+ULONG WINAPI EventActivityIdControl(ULONG code, GUID *guid)
+{
+    FIXME("0x%x, %p: stub\n", code, guid);
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * EventWrite [ADVAPI32.@]
  */
@@ -917,6 +938,16 @@ ULONG WINAPI EventWrite( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, ULONG
     return ERROR_SUCCESS;
 }
 
+/******************************************************************************
+ * EventSetInformation [ADVAPI32.@]
+ */
+ULONG WINAPI EventSetInformation( REGHANDLE handle, EVENT_INFO_CLASS class, PVOID info,
+                                  ULONG length )
+{
+    FIXME("%u, %p, %u\n", class, info, length);
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * QueryTraceW [ADVAPI32.@]
  */
@@ -997,3 +1028,12 @@ ULONG WINAPI EnumerateTraceGuids(PTRACE_GUID_PROPERTIES *propertiesarray,
     FIXME("%p %d %p: stub\n", propertiesarray, arraycount, guidcount);
     return ERROR_INVALID_PARAMETER;
 }
+
+/******************************************************************************
+ * WmiOpenBlock [ADVAPI32.@]
+ */
+NTSTATUS WINAPI WmiOpenBlock(GUID *guid, ULONG access, PVOID *datablock)
+{
+    FIXME("%s %d %p: stub\n", debugstr_guid(guid), access, datablock);
+    return ERROR_SUCCESS;
+}
