@@ -24,6 +24,12 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d10core);
 
+#define D3D10_FILTER_MIP_MASK       0x01
+#define D3D10_FILTER_MAG_MASK       0x04
+#define D3D10_FILTER_MIN_MASK       0x10
+#define D3D10_FILTER_ANISO_MASK     0x40
+#define D3D10_FILTER_COMPARE_MASK   0x80
+
 static inline struct d3d10_blend_state *impl_from_ID3D10BlendState(ID3D10BlendState *iface)
 {
     return CONTAINING_RECORD(iface, struct d3d10_blend_state, ID3D10BlendState_iface);
@@ -73,6 +79,7 @@ static ULONG STDMETHODCALLTYPE d3d10_blend_state_Release(ID3D10BlendState *iface
         struct d3d10_device *device = impl_from_ID3D10Device(state->device);
         wine_rb_remove(&device->blend_states, &state->desc);
         ID3D10Device1_Release(state->device);
+        wined3d_private_store_cleanup(&state->private_store);
         HeapFree(GetProcessHeap(), 0, state);
     }
 
@@ -94,27 +101,33 @@ static void STDMETHODCALLTYPE d3d10_blend_state_GetDevice(ID3D10BlendState *ifac
 static HRESULT STDMETHODCALLTYPE d3d10_blend_state_GetPrivateData(ID3D10BlendState *iface,
         REFGUID guid, UINT *data_size, void *data)
 {
-    FIXME("iface %p, guid %s, data_size %p, data %p stub!\n",
+    struct d3d10_blend_state *state = impl_from_ID3D10BlendState(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_get_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_blend_state_SetPrivateData(ID3D10BlendState *iface,
         REFGUID guid, UINT data_size, const void *data)
 {
-    FIXME("iface %p, guid %s, data_size %u, data %p stub!\n",
+    struct d3d10_blend_state *state = impl_from_ID3D10BlendState(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_set_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_blend_state_SetPrivateDataInterface(ID3D10BlendState *iface,
         REFGUID guid, const IUnknown *data)
 {
-    FIXME("iface %p, guid %s, data %p stub!\n", iface, debugstr_guid(guid), data);
+    struct d3d10_blend_state *state = impl_from_ID3D10BlendState(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d10_set_private_data_interface(&state->private_store, guid, data);
 }
 
 /* ID3D10BlendState methods */
@@ -149,11 +162,13 @@ HRESULT d3d10_blend_state_init(struct d3d10_blend_state *state, struct d3d10_dev
 {
     state->ID3D10BlendState_iface.lpVtbl = &d3d10_blend_state_vtbl;
     state->refcount = 1;
+    wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
     if (wine_rb_put(&device->blend_states, desc, &state->entry) == -1)
     {
         ERR("Failed to insert blend state entry.\n");
+        wined3d_private_store_cleanup(&state->private_store);
         return E_FAIL;
     }
 
@@ -221,6 +236,7 @@ static ULONG STDMETHODCALLTYPE d3d10_depthstencil_state_Release(ID3D10DepthStenc
         struct d3d10_device *device = impl_from_ID3D10Device(state->device);
         wine_rb_remove(&device->depthstencil_states, &state->desc);
         ID3D10Device1_Release(state->device);
+        wined3d_private_store_cleanup(&state->private_store);
         HeapFree(GetProcessHeap(), 0, state);
     }
 
@@ -242,27 +258,33 @@ static void STDMETHODCALLTYPE d3d10_depthstencil_state_GetDevice(ID3D10DepthSten
 static HRESULT STDMETHODCALLTYPE d3d10_depthstencil_state_GetPrivateData(ID3D10DepthStencilState *iface,
         REFGUID guid, UINT *data_size, void *data)
 {
-    FIXME("iface %p, guid %s, data_size %p, data %p stub!\n",
+    struct d3d10_depthstencil_state *state = impl_from_ID3D10DepthStencilState(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_get_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_depthstencil_state_SetPrivateData(ID3D10DepthStencilState *iface,
         REFGUID guid, UINT data_size, const void *data)
 {
-    FIXME("iface %p, guid %s, data_size %u, data %p stub!\n",
+    struct d3d10_depthstencil_state *state = impl_from_ID3D10DepthStencilState(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_set_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_depthstencil_state_SetPrivateDataInterface(ID3D10DepthStencilState *iface,
         REFGUID guid, const IUnknown *data)
 {
-    FIXME("iface %p, guid %s, data %p stub!\n", iface, debugstr_guid(guid), data);
+    struct d3d10_depthstencil_state *state = impl_from_ID3D10DepthStencilState(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d10_set_private_data_interface(&state->private_store, guid, data);
 }
 
 /* ID3D10DepthStencilState methods */
@@ -297,11 +319,13 @@ HRESULT d3d10_depthstencil_state_init(struct d3d10_depthstencil_state *state, st
 {
     state->ID3D10DepthStencilState_iface.lpVtbl = &d3d10_depthstencil_state_vtbl;
     state->refcount = 1;
+    wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
     if (wine_rb_put(&device->depthstencil_states, desc, &state->entry) == -1)
     {
         ERR("Failed to insert depthstencil state entry.\n");
+        wined3d_private_store_cleanup(&state->private_store);
         return E_FAIL;
     }
 
@@ -369,6 +393,7 @@ static ULONG STDMETHODCALLTYPE d3d10_rasterizer_state_Release(ID3D10RasterizerSt
         struct d3d10_device *device = impl_from_ID3D10Device(state->device);
         wine_rb_remove(&device->rasterizer_states, &state->desc);
         ID3D10Device1_Release(state->device);
+        wined3d_private_store_cleanup(&state->private_store);
         HeapFree(GetProcessHeap(), 0, state);
     }
 
@@ -390,27 +415,33 @@ static void STDMETHODCALLTYPE d3d10_rasterizer_state_GetDevice(ID3D10RasterizerS
 static HRESULT STDMETHODCALLTYPE d3d10_rasterizer_state_GetPrivateData(ID3D10RasterizerState *iface,
         REFGUID guid, UINT *data_size, void *data)
 {
-    FIXME("iface %p, guid %s, data_size %p, data %p stub!\n",
+    struct d3d10_rasterizer_state *state = impl_from_ID3D10RasterizerState(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_get_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_rasterizer_state_SetPrivateData(ID3D10RasterizerState *iface,
         REFGUID guid, UINT data_size, const void *data)
 {
-    FIXME("iface %p, guid %s, data_size %u, data %p stub!\n",
+    struct d3d10_rasterizer_state *state = impl_from_ID3D10RasterizerState(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_set_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_rasterizer_state_SetPrivateDataInterface(ID3D10RasterizerState *iface,
         REFGUID guid, const IUnknown *data)
 {
-    FIXME("iface %p, guid %s, data %p stub!\n", iface, debugstr_guid(guid), data);
+    struct d3d10_rasterizer_state *state = impl_from_ID3D10RasterizerState(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d10_set_private_data_interface(&state->private_store, guid, data);
 }
 
 /* ID3D10RasterizerState methods */
@@ -445,11 +476,13 @@ HRESULT d3d10_rasterizer_state_init(struct d3d10_rasterizer_state *state, struct
 {
     state->ID3D10RasterizerState_iface.lpVtbl = &d3d10_rasterizer_state_vtbl;
     state->refcount = 1;
+    wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
     if (wine_rb_put(&device->rasterizer_states, desc, &state->entry) == -1)
     {
         ERR("Failed to insert rasterizer state entry.\n");
+        wined3d_private_store_cleanup(&state->private_store);
         return E_FAIL;
     }
 
@@ -519,6 +552,7 @@ static ULONG STDMETHODCALLTYPE d3d10_sampler_state_Release(ID3D10SamplerState *i
         wined3d_sampler_decref(state->wined3d_sampler);
         wine_rb_remove(&device->sampler_states, &state->desc);
         ID3D10Device1_Release(state->device);
+        wined3d_private_store_cleanup(&state->private_store);
         HeapFree(GetProcessHeap(), 0, state);
     }
 
@@ -540,27 +574,33 @@ static void STDMETHODCALLTYPE d3d10_sampler_state_GetDevice(ID3D10SamplerState *
 static HRESULT STDMETHODCALLTYPE d3d10_sampler_state_GetPrivateData(ID3D10SamplerState *iface,
         REFGUID guid, UINT *data_size, void *data)
 {
-    FIXME("iface %p, guid %s, data_size %p, data %p stub!\n",
+    struct d3d10_sampler_state *state = impl_from_ID3D10SamplerState(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_get_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_sampler_state_SetPrivateData(ID3D10SamplerState *iface,
         REFGUID guid, UINT data_size, const void *data)
 {
-    FIXME("iface %p, guid %s, data_size %u, data %p stub!\n",
+    struct d3d10_sampler_state *state = impl_from_ID3D10SamplerState(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    return d3d10_set_private_data(&state->private_store, guid, data_size, data);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_sampler_state_SetPrivateDataInterface(ID3D10SamplerState *iface,
         REFGUID guid, const IUnknown *data)
 {
-    FIXME("iface %p, guid %s, data %p stub!\n", iface, debugstr_guid(guid), data);
+    struct d3d10_sampler_state *state = impl_from_ID3D10SamplerState(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d10_set_private_data_interface(&state->private_store, guid, data);
 }
 
 /* ID3D10SamplerState methods */
@@ -590,18 +630,72 @@ static const struct ID3D10SamplerStateVtbl d3d10_sampler_state_vtbl =
     d3d10_sampler_state_GetDesc,
 };
 
+static enum wined3d_texture_address wined3d_texture_address_from_d3d10core(enum D3D10_TEXTURE_ADDRESS_MODE t)
+{
+    return (enum wined3d_texture_address)t;
+}
+
+static enum wined3d_texture_filter_type wined3d_texture_filter_mip_from_d3d10core(enum D3D10_FILTER f)
+{
+    if (f & D3D10_FILTER_MIP_MASK)
+        return WINED3D_TEXF_LINEAR;
+    return WINED3D_TEXF_POINT;
+}
+
+static enum wined3d_texture_filter_type wined3d_texture_filter_mag_from_d3d10core(enum D3D10_FILTER f)
+{
+    if (f & D3D10_FILTER_MAG_MASK)
+        return WINED3D_TEXF_LINEAR;
+    return WINED3D_TEXF_POINT;
+}
+
+static enum wined3d_texture_filter_type wined3d_texture_filter_min_from_d3d10core(enum D3D10_FILTER f)
+{
+    if (f & D3D10_FILTER_MIN_MASK)
+        return WINED3D_TEXF_LINEAR;
+    return WINED3D_TEXF_POINT;
+}
+
+static BOOL wined3d_texture_compare_from_d3d10core(enum D3D10_FILTER f)
+{
+    return f & D3D10_FILTER_COMPARE_MASK;
+}
+
+static enum wined3d_cmp_func wined3d_cmp_func_from_d3d10core(D3D10_COMPARISON_FUNC f)
+{
+    return (enum wined3d_cmp_func)f;
+}
+
 HRESULT d3d10_sampler_state_init(struct d3d10_sampler_state *state, struct d3d10_device *device,
         const D3D10_SAMPLER_DESC *desc)
 {
+    struct wined3d_sampler_desc wined3d_desc;
     HRESULT hr;
 
     state->ID3D10SamplerState_iface.lpVtbl = &d3d10_sampler_state_vtbl;
     state->refcount = 1;
+    wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
-    if (FAILED(hr = wined3d_sampler_create(state, &state->wined3d_sampler)))
+    wined3d_desc.address_u = wined3d_texture_address_from_d3d10core(desc->AddressU);
+    wined3d_desc.address_v = wined3d_texture_address_from_d3d10core(desc->AddressV);
+    wined3d_desc.address_w = wined3d_texture_address_from_d3d10core(desc->AddressW);
+    memcpy(wined3d_desc.border_color, desc->BorderColor, sizeof(wined3d_desc.border_color));
+    wined3d_desc.mag_filter = wined3d_texture_filter_mag_from_d3d10core(desc->Filter);
+    wined3d_desc.min_filter = wined3d_texture_filter_min_from_d3d10core(desc->Filter);
+    wined3d_desc.mip_filter = wined3d_texture_filter_mip_from_d3d10core(desc->Filter);
+    wined3d_desc.lod_bias = desc->MipLODBias;
+    wined3d_desc.min_lod = desc->MinLOD;
+    wined3d_desc.max_lod = desc->MaxLOD;
+    wined3d_desc.max_anisotropy = desc->Filter & D3D10_FILTER_ANISO_MASK ? desc->MaxAnisotropy : 1;
+    wined3d_desc.compare = wined3d_texture_compare_from_d3d10core(desc->Filter);
+    wined3d_desc.comparison_func = wined3d_cmp_func_from_d3d10core(desc->ComparisonFunc);
+    wined3d_desc.srgb_decode = FALSE;
+
+    if (FAILED(hr = wined3d_sampler_create(device->wined3d_device, &wined3d_desc, state, &state->wined3d_sampler)))
     {
         WARN("Failed to create wined3d sampler, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&state->private_store);
         return hr;
     }
 
@@ -609,6 +703,7 @@ HRESULT d3d10_sampler_state_init(struct d3d10_sampler_state *state, struct d3d10
     {
         ERR("Failed to insert sampler state entry.\n");
         wined3d_sampler_decref(state->wined3d_sampler);
+        wined3d_private_store_cleanup(&state->private_store);
         return E_FAIL;
     }
 
