@@ -1,7 +1,8 @@
 /*
- * D3DX10 main file
+ * Direct3D X 9 main file
  *
- * Copyright (c) 2010 Owen Rudge for CodeWeavers
+ * Copyright (C) 2007 David Adam
+ * Copyright (C) 2008 Tony Wasserka
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,41 +22,59 @@
 
 #include "config.h"
 #include "wine/port.h"
+#include "initguid.h"
 
 #include <stdarg.h>
 
-#define COBJMACROS
-
 #include "windef.h"
 #include "winbase.h"
+#include "wingdi.h"
 #include "winuser.h"
-#include "objbase.h"
 
-#include "d3d10.h"
+#include "d3dx9.h"
 
-BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+/***********************************************************************
+ * DllMain.
+ */
+BOOL WINAPI DllMain(HINSTANCE inst, DWORD reason, void *reserved)
 {
-    switch (fdwReason)
+    switch(reason)
     {
-        case DLL_WINE_PREATTACH:
-            return FALSE;    /* prefer native version */
-        case DLL_PROCESS_ATTACH:
-            DisableThreadLibraryCalls(hinstDLL);
-            break;
+    case DLL_WINE_PREATTACH:
+        return FALSE; /* prefer native version */
+    case DLL_PROCESS_ATTACH:
+        DisableThreadLibraryCalls(inst);
+        break;
     }
-
-   return TRUE;
+    return TRUE;
 }
 
 /***********************************************************************
- * D3DX10CheckVersion
- *
+ * D3DXCheckVersion
  * Checks whether we are compiling against the correct d3d and d3dx library.
  */
-BOOL WINAPI D3DX10CheckVersion(UINT d3dsdkvers, UINT d3dxsdkvers)
+BOOL WINAPI D3DXCheckVersion(UINT d3dsdkvers, UINT d3dxsdkvers)
 {
-    if ((d3dsdkvers == D3D10_SDK_VERSION) && (d3dxsdkvers == 42))
+    if(d3dsdkvers==D3D_SDK_VERSION && d3dxsdkvers==36)
         return TRUE;
+    else
+        return FALSE;
+}
 
+/***********************************************************************
+ * D3DXDebugMute
+ * Returns always FALSE for us.
+ */
+BOOL WINAPI D3DXDebugMute(BOOL mute)
+{
     return FALSE;
 }
+
+/***********************************************************************
+ * D3DXGetDriverLevel.
+ * Returns always 900 (DX 9) for us
+ */
+UINT WINAPI D3DXGetDriverLevel(struct IDirect3DDevice9 *device)
+{
+    return 900;
+}
