@@ -1,7 +1,7 @@
 /*
  *    FreeType integration
  *
- * Copyright 2014 Nikolay Sivov for CodeWeavers
+ * Copyright 2014-2015 Nikolay Sivov for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 #include FT_CACHE_H
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
+#include FT_TRUETYPE_TABLES_H
 #endif /* HAVE_FT2BUILD_H */
 
 #include "windef.h"
@@ -54,6 +55,7 @@ static void *ft_handle = NULL;
 static FT_Library library = 0;
 static FTC_Manager cache_manager = 0;
 static FTC_CMapCache cmap_cache = 0;
+static FTC_ImageCache image_cache = 0;
 typedef struct
 {
     FT_Int major;
@@ -63,14 +65,24 @@ typedef struct
 
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f = NULL
 MAKE_FUNCPTR(FT_Done_FreeType);
+MAKE_FUNCPTR(FT_Get_First_Char);
 MAKE_FUNCPTR(FT_Get_Kerning);
+MAKE_FUNCPTR(FT_Get_Sfnt_Table);
+MAKE_FUNCPTR(FT_Glyph_Get_CBox);
 MAKE_FUNCPTR(FT_Init_FreeType);
 MAKE_FUNCPTR(FT_Library_Version);
 MAKE_FUNCPTR(FT_Load_Glyph);
 MAKE_FUNCPTR(FT_New_Memory_Face);
+MAKE_FUNCPTR(FT_Outline_Copy);
+MAKE_FUNCPTR(FT_Outline_Done);
+MAKE_FUNCPTR(FT_Outline_Get_Bitmap);
+MAKE_FUNCPTR(FT_Outline_New);
 MAKE_FUNCPTR(FT_Outline_Transform);
+MAKE_FUNCPTR(FT_Outline_Translate);
 MAKE_FUNCPTR(FTC_CMapCache_Lookup);
 MAKE_FUNCPTR(FTC_CMapCache_New);
+MAKE_FUNCPTR(FTC_ImageCache_Lookup);
+MAKE_FUNCPTR(FTC_ImageCache_New);
 MAKE_FUNCPTR(FTC_Manager_New);
 MAKE_FUNCPTR(FTC_Manager_Done);
 MAKE_FUNCPTR(FTC_Manager_LookupFace);
@@ -136,14 +148,24 @@ BOOL init_freetype(void)
 
 #define LOAD_FUNCPTR(f) if((p##f = wine_dlsym(ft_handle, #f, NULL, 0)) == NULL){WARN("Can't find symbol %s\n", #f); goto sym_not_found;}
     LOAD_FUNCPTR(FT_Done_FreeType)
+    LOAD_FUNCPTR(FT_Get_First_Char)
     LOAD_FUNCPTR(FT_Get_Kerning)
+    LOAD_FUNCPTR(FT_Get_Sfnt_Table)
+    LOAD_FUNCPTR(FT_Glyph_Get_CBox)
     LOAD_FUNCPTR(FT_Init_FreeType)
     LOAD_FUNCPTR(FT_Library_Version)
     LOAD_FUNCPTR(FT_Load_Glyph)
     LOAD_FUNCPTR(FT_New_Memory_Face)
+    LOAD_FUNCPTR(FT_Outline_Copy)
+    LOAD_FUNCPTR(FT_Outline_Done)
+    LOAD_FUNCPTR(FT_Outline_Get_Bitmap)
+    LOAD_FUNCPTR(FT_Outline_New)
     LOAD_FUNCPTR(FT_Outline_Transform)
+    LOAD_FUNCPTR(FT_Outline_Translate)
     LOAD_FUNCPTR(FTC_CMapCache_Lookup)
     LOAD_FUNCPTR(FTC_CMapCache_New)
+    LOAD_FUNCPTR(FTC_ImageCache_Lookup)
+    LOAD_FUNCPTR(FTC_ImageCache_New)
     LOAD_FUNCPTR(FTC_Manager_New)
     LOAD_FUNCPTR(FTC_Manager_Done)
     LOAD_FUNCPTR(FTC_Manager_LookupFace)
@@ -161,7 +183,8 @@ BOOL init_freetype(void)
 
     /* init cache manager */
     if (pFTC_Manager_New(library, 0, 0, 0, &face_requester, NULL, &cache_manager) != 0 ||
-        pFTC_CMapCache_New(cache_manager, &cmap_cache) != 0) {
+        pFTC_CMapCache_New(cache_manager, &cmap_cache) != 0 ||
+        pFTC_ImageCache_New(cache_manager, &image_cache) != 0) {
 
         ERR("Failed to init FreeType cache\n");
         pFTC_Manager_Done(cache_manager);
@@ -414,12 +437,20 @@ UINT16 freetype_get_glyphcount(IDWriteFontFace2 *fontface)
     return count;
 }
 
-UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint)
+UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint, INT charmap)
 {
     UINT16 glyph;
 
     EnterCriticalSection(&freetype_cs);
-    glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, -1, codepoint);
+    if (charmap == -1)
+        glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint);
+    else {
+        /* special handling for symbol fonts */
+        if (codepoint < 0x100) codepoint += 0xf000;
+        glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint);
+        if (!glyph)
+            glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint - 0xf000);
+    }
     LeaveCriticalSection(&freetype_cs);
 
     return glyph;
@@ -456,6 +487,118 @@ INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 le
     return adjustment;
 }
 
+void freetype_get_glyph_bbox(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, BOOL nohint, RECT *ret)
+{
+    FTC_ImageTypeRec imagetype;
+    FT_BBox bbox = { 0 };
+    FT_Glyph glyph;
+
+    imagetype.face_id = fontface;
+    imagetype.width = 0;
+    imagetype.height = emSize;
+    imagetype.flags = FT_LOAD_DEFAULT;
+
+    EnterCriticalSection(&freetype_cs);
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0)
+        pFT_Glyph_Get_CBox(glyph, FT_GLYPH_BBOX_PIXELS, &bbox);
+    LeaveCriticalSection(&freetype_cs);
+
+    /* flip Y axis */
+    ret->left = bbox.xMin;
+    ret->right = bbox.xMax;
+    ret->top = -bbox.yMax;
+    ret->bottom = -bbox.yMin;
+}
+
+void freetype_get_glyph_bitmap(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, const RECT *bbox, BYTE *buf)
+{
+    FTC_ImageTypeRec imagetype;
+    FT_Glyph glyph;
+
+    imagetype.face_id = fontface;
+    imagetype.width = 0;
+    imagetype.height = emSize;
+    imagetype.flags = FT_LOAD_DEFAULT;
+
+    EnterCriticalSection(&freetype_cs);
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0) {
+        int width = bbox->right - bbox->left;
+        int pitch = ((width + 31) >> 5) << 2;
+        int height = bbox->bottom - bbox->top;
+
+        if (glyph->format == FT_GLYPH_FORMAT_OUTLINE) {
+            FT_OutlineGlyph outline = (FT_OutlineGlyph)glyph;
+            const FT_Outline *src = &outline->outline;
+            FT_Bitmap ft_bitmap;
+            FT_Outline copy;
+
+            ft_bitmap.width = width;
+            ft_bitmap.rows = height;
+            ft_bitmap.pitch = pitch;
+            ft_bitmap.pixel_mode = FT_PIXEL_MODE_MONO;
+            ft_bitmap.buffer = buf;
+
+            /* Note: FreeType will only set 'black' bits for us. */
+            memset(buf, 0, height*pitch);
+            if (pFT_Outline_New(library, src->n_points, src->n_contours, &copy) == 0) {
+                pFT_Outline_Copy(src, &copy);
+                pFT_Outline_Translate(&copy, -bbox->left << 6, bbox->bottom << 6);
+                pFT_Outline_Get_Bitmap(library, &copy, &ft_bitmap);
+                pFT_Outline_Done(library, &copy);
+            }
+        }
+        else if (glyph->format == FT_GLYPH_FORMAT_BITMAP) {
+            FT_Bitmap *bitmap = &((FT_BitmapGlyph)glyph)->bitmap;
+            BYTE *src = bitmap->buffer, *dst = buf;
+            int w = min(pitch, (bitmap->width + 7) >> 3);
+            int h = min(height, bitmap->rows);
+
+            memset(buf, 0, height*pitch);
+
+            while (h--) {
+                memcpy(dst, src, w);
+                src += bitmap->pitch;
+                dst += pitch;
+            }
+        }
+        else
+            FIXME("format %d not handled\n", glyph->format);
+    }
+    LeaveCriticalSection(&freetype_cs);
+}
+
+INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
+{
+    INT charmap_index = -1;
+    FT_Face face;
+
+    *is_symbol = FALSE;
+
+    EnterCriticalSection(&freetype_cs);
+    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0) {
+        TT_OS2 *os2 = pFT_Get_Sfnt_Table(face, ft_sfnt_os2);
+        FT_Int i;
+
+        if (os2) {
+            FT_UInt dummy;
+            if (os2->version == 0)
+                *is_symbol = pFT_Get_First_Char(face, &dummy) >= 0x100;
+            else
+                *is_symbol = os2->ulCodePageRange1 & FS_SYMBOL;
+        }
+
+        for (i = 0; i < face->num_charmaps; i++)
+            if (face->charmaps[i]->encoding == FT_ENCODING_MS_SYMBOL) {
+                *is_symbol = TRUE;
+                charmap_index = i;
+                break;
+            }
+    }
+    LeaveCriticalSection(&freetype_cs);
+
+    return charmap_index;
+}
+
 #else /* HAVE_FREETYPE */
 
 BOOL init_freetype(void)
@@ -492,7 +635,7 @@ UINT16 freetype_get_glyphcount(IDWriteFontFace2 *fontface)
     return 0;
 }
 
-UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint)
+UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint, INT charmap)
 {
     return 0;
 }
@@ -507,4 +650,21 @@ INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 le
     return 0;
 }
 
+void freetype_get_glyph_bbox(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, BOOL nohint, RECT *ret)
+{
+    ret->left = ret->right = ret->top = ret->bottom = 0;
+}
+
+void freetype_get_glyph_bitmap(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, const RECT *bbox, BYTE *buf)
+{
+    UINT32 size = (bbox->right - bbox->left)*(bbox->bottom - bbox->top);
+    memset(buf, 0, size);
+}
+
+INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
+{
+    *is_symbol = FALSE;
+    return -1;
+}
+
 #endif /* HAVE_FREETYPE */
