@@ -180,7 +180,7 @@ static HRESULT GAMEUX_updateStatisticsFile(struct GAMEUX_STATS *stats)
         {
             IXMLDOMElement *categoryElement = NULL;
 
-            if(lstrlenW(stats->categories[i].sName)==0)
+            if(!stats->categories[i].sName[0])
                 continue;
 
             V_VT(&vValue) = VT_INT;
@@ -218,7 +218,7 @@ static HRESULT GAMEUX_updateStatisticsFile(struct GAMEUX_STATS *stats)
             {
                 for(j=0; j<MAX_STATS_PER_CATEGORY; ++j)
                 {
-                    if(lstrlenW(stats->categories[i].stats[j].sName)==0)
+                    if(!stats->categories[i].stats[j].sName[0])
                         continue;
 
                     V_VT(&vValue) = VT_INT;
@@ -627,6 +627,7 @@ static HRESULT GAMEUX_loadGameStatistics(struct GAMEUX_STATS *pStats,
     TRACE("(%p, %s, %d, %p)\n", pStats, debugstr_w(sGameId), openType, pOpenResult);
 
     hr = GAMEUX_buildStatisticsFilePath(sGameId, pStats->sStatsFile);
+    if (FAILED(hr)) return hr;
 
     hr = GAMEUX_loadStatisticsFromFile(pStats);
     TRACE("ldstats finished, res: %#x\n", hr);
@@ -1127,7 +1128,7 @@ static HRESULT STDMETHODCALLTYPE GameStatisticsMgrImpl_RemoveGameStatistics(
         hr = GAMEUX_buildStatisticsFilePath(lpApplicationId, sStatsFile);
 
     if(SUCCEEDED(hr))
-        hr = (DeleteFileW(sStatsFile)==TRUE ? S_OK : HRESULT_FROM_WIN32(GetLastError()));
+        hr = DeleteFileW(sStatsFile) ? S_OK : HRESULT_FROM_WIN32(GetLastError());
 
     return hr;
 }
