@@ -23,7 +23,6 @@
 
 #define COBJMACROS
 #define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 
 #include "windef.h"
 #include "winbase.h"
@@ -213,6 +212,8 @@ static nsProtocolStream *create_nsprotocol_stream(void)
 
 static void release_request_data(request_data_t *request_data)
 {
+    if(request_data->post_stream)
+        nsIInputStream_Release(request_data->post_stream);
     heap_free(request_data->headers);
     if(request_data->post_data)
         GlobalFree(request_data->post_data);
@@ -801,7 +802,7 @@ HRESULT start_binding(HTMLInnerWindow *inner_window, BSCallback *bscallback, IBi
     }
 
     if(FAILED(hres)) {
-        bscallback->vtbl->stop_binding(bscallback, hres);
+        bscallback->window = NULL;
         return hres;
     }
 
@@ -809,7 +810,7 @@ HRESULT start_binding(HTMLInnerWindow *inner_window, BSCallback *bscallback, IBi
     IBindCtx_Release(bctx);
     if(FAILED(hres)) {
         WARN("BindToStorage failed: %08x\n", hres);
-        bscallback->vtbl->stop_binding(bscallback, hres);
+        bscallback->window = NULL;
         return hres;
     }
 
@@ -822,6 +823,7 @@ HRESULT start_binding(HTMLInnerWindow *inner_window, BSCallback *bscallback, IBi
 static HRESULT read_post_data_stream(nsIInputStream *stream, BOOL contains_headers, struct list *headers_list,
         request_data_t *request_data)
 {
+    nsISeekableStream *seekable_stream;
     UINT64 available = 0;
     UINT32 data_len = 0;
     char *data, *post_data;
@@ -902,9 +904,21 @@ static HRESULT read_post_data_stream(nsIInputStream *stream, BOOL contains_heade
         post_data = new_data;
     }
 
-    post_data[data_len] = 0;
+    if(post_data)
+        post_data[data_len] = 0;
     request_data->post_data = post_data;
     request_data->post_data_len = data_len;
+
+    nsres = nsIInputStream_QueryInterface(stream, &IID_nsISeekableStream, (void**)&seekable_stream);
+    assert(nsres == NS_OK);
+
+    nsres = nsISeekableStream_Seek(seekable_stream, NS_SEEK_SET, 0);
+    assert(nsres == NS_OK);
+
+    nsISeekableStream_Release(seekable_stream);
+
+    nsIInputStream_AddRef(stream);
+    request_data->post_stream = stream;
     TRACE("post_data = %s\n", debugstr_an(request_data->post_data, request_data->post_data_len));
     return S_OK;
 }
@@ -1581,10 +1595,29 @@ static HRESULT nsChannelBSC_on_progress(BSCallback *bsc, ULONG status_code, LPCW
     return S_OK;
 }
 
+static HRESULT process_response_status_text(const WCHAR *header, const WCHAR *header_end, char **status_text)
+{
+    header = strchrW(header + 1, ' ');
+    if(!header || header >= header_end)
+        return E_FAIL;
+    header = strchrW(header + 1, ' ');
+    if(!header || header >= header_end)
+        return E_FAIL;
+    ++header;
+
+    *status_text = heap_strndupWtoU(header, header_end - header);
+
+    if(!*status_text)
+        return E_OUTOFMEMORY;
+
+    return S_OK;
+}
+
 static HRESULT nsChannelBSC_on_response(BSCallback *bsc, DWORD response_code,
         LPCWSTR response_headers)
 {
     nsChannelBSC *This = nsChannelBSC_from_BSCallback(bsc);
+    char *str;
     HRESULT hres;
 
     This->response_processed = TRUE;
@@ -1594,6 +1627,15 @@ static HRESULT nsChannelBSC_on_response(BSCallback *bsc, DWORD response_code,
         const WCHAR *headers;
 
         headers = strchrW(response_headers, '\r');
+        hres = process_response_status_text(response_headers, headers, &str);
+        if(FAILED(hres)) {
+            WARN("parsing headers failed: %08x\n", hres);
+            return hres;
+        }
+
+        heap_free(This->nschannel->response_status_text);
+        This->nschannel->response_status_text = str;
+
         if(headers && headers[1] == '\n') {
             headers += 2;
             hres = process_response_headers(This, headers);
@@ -1964,7 +2006,7 @@ static HRESULT navigate_fragment(HTMLOuterWindow *window, IUri *uri)
         sprintfW(selector, selector_formatW, frag);
         nsAString_InitDepend(&selector_str, selector);
         /* NOTE: Gecko doesn't set result to NULL if there is no match, so nselem must be initialized */
-        nsres = nsIDOMNodeSelector_QuerySelector(window->base.inner_window->doc->nsnode_selector, &selector_str, &nselem);
+        nsres = nsIDOMHTMLDocument_QuerySelector(window->base.inner_window->doc->nsdoc, &selector_str, &nselem);
         nsAString_Finish(&selector_str);
         heap_free(selector);
         if(NS_SUCCEEDED(nsres) && nselem) {
@@ -2106,7 +2148,7 @@ HRESULT super_navigate(HTMLOuterWindow *window, IUri *uri, DWORD flags, const WC
     return hres;
 }
 
-HRESULT navigate_new_window(HTMLOuterWindow *window, IUri *uri, const WCHAR *name, IHTMLWindow2 **ret)
+HRESULT navigate_new_window(HTMLOuterWindow *window, IUri *uri, const WCHAR *name, request_data_t *request_data, IHTMLWindow2 **ret)
 {
     IWebBrowser2 *web_browser;
     IHTMLWindow2 *new_window;
@@ -2114,7 +2156,12 @@ HRESULT navigate_new_window(HTMLOuterWindow *window, IUri *uri, const WCHAR *nam
     nsChannelBSC *bsc;
     HRESULT hres;
 
-    hres = create_channelbsc(NULL, NULL, NULL, 0, FALSE, &bsc);
+    if(request_data)
+        hres = create_channelbsc(NULL, request_data->headers,
+                request_data->post_data, request_data->post_data_len, FALSE,
+                &bsc);
+    else
+        hres = create_channelbsc(NULL, NULL, NULL, 0, FALSE, &bsc);
     if(FAILED(hres))
         return hres;
 
@@ -2225,23 +2272,32 @@ static HRESULT navigate_uri(HTMLOuterWindow *window, IUri *uri, const WCHAR *dis
 
     TRACE("%s\n", debugstr_w(display_uri));
 
-    if(window->doc_obj && window->doc_obj->webbrowser && window == window->doc_obj->basedoc.window) {
+    if(window->doc_obj && window->doc_obj->webbrowser) {
         DWORD post_data_len = request_data ? request_data->post_data_len : 0;
         void *post_data = post_data_len ? request_data->post_data : NULL;
         const WCHAR *headers = request_data ? request_data->headers : NULL;
 
         if(!(flags & BINDING_REFRESH)) {
+            BSTR frame_name = NULL;
             BOOL cancel = FALSE;
 
+            if(window != window->doc_obj->basedoc.window) {
+                hres = IHTMLWindow2_get_name(&window->base.IHTMLWindow2_iface, &frame_name);
+                if(FAILED(hres))
+                    return hres;
+            }
+
             hres = IDocObjectService_FireBeforeNavigate2(window->doc_obj->doc_object_service, NULL, display_uri, 0x40,
-                    NULL, post_data, post_data_len ? post_data_len+1 : 0, headers, TRUE, &cancel);
+                    frame_name, post_data, post_data_len ? post_data_len+1 : 0, headers, TRUE, &cancel);
+            SysFreeString(frame_name);
             if(SUCCEEDED(hres) && cancel) {
                 TRACE("Navigation canceled\n");
                 return S_OK;
             }
         }
 
-        return super_navigate(window, uri, flags, headers, post_data, post_data_len);
+        if(window == window->doc_obj->basedoc.window)
+            return super_navigate(window, uri, flags, headers, post_data, post_data_len);
     }
 
     if(window->doc_obj && window == window->doc_obj->basedoc.window) {
@@ -2261,7 +2317,7 @@ static HRESULT navigate_uri(HTMLOuterWindow *window, IUri *uri, const WCHAR *dis
     if(FAILED(hres))
         return hres;
 
-    hres = load_nsuri(window, nsuri, NULL, LOAD_FLAGS_NONE);
+    hres = load_nsuri(window, nsuri, request_data ? request_data->post_stream : NULL, NULL, LOAD_FLAGS_NONE);
     nsISupports_Release((nsISupports*)nsuri);
     return hres;
 }
@@ -2295,7 +2351,7 @@ static HRESULT translate_uri(HTMLOuterWindow *window, IUri *orig_uri, BSTR *ret_
 
         hres = IDocHostUIHandler_TranslateUrl(window->doc_obj->hostui, 0, display_uri,
                 &translated_url);
-        if(hres == S_OK) {
+        if(hres == S_OK && translated_url) {
             TRACE("%08x %s -> %s\n", hres, debugstr_w(display_uri), debugstr_w(translated_url));
             SysFreeString(display_uri);
             hres = create_uri(translated_url, 0, &uri);
@@ -2321,23 +2377,32 @@ static HRESULT translate_uri(HTMLOuterWindow *window, IUri *orig_uri, BSTR *ret_
     return S_OK;
 }
 
-HRESULT submit_form(HTMLOuterWindow *window, IUri *submit_uri, nsIInputStream *post_stream)
+HRESULT submit_form(HTMLOuterWindow *window, const WCHAR *target, IUri *submit_uri, nsIInputStream *post_stream)
 {
     request_data_t request_data = {NULL};
-    BSTR display_uri;
-    IUri *uri;
     HRESULT hres;
 
     hres = read_post_data_stream(post_stream, TRUE, NULL, &request_data);
     if(FAILED(hres))
         return hres;
 
-    hres = translate_uri(window, submit_uri, &display_uri, &uri);
-    if(SUCCEEDED(hres)) {
-        hres = navigate_uri(window, uri, display_uri, &request_data, BINDING_NAVIGATED|BINDING_SUBMIT);
-        IUri_Release(uri);
-        SysFreeString(display_uri);
-    }
+    if(window) {
+        IUri *uri;
+        BSTR display_uri;
+
+        window->readystate_locked++;
+
+        hres = translate_uri(window, submit_uri, &display_uri, &uri);
+        if(SUCCEEDED(hres)) {
+            hres = navigate_uri(window, uri, display_uri, &request_data, BINDING_NAVIGATED|BINDING_SUBMIT);
+            IUri_Release(uri);
+            SysFreeString(display_uri);
+        }
+
+        window->readystate_locked--;
+    }else
+        hres = navigate_new_window(window, submit_uri, target, &request_data, NULL);
+
     release_request_data(&request_data);
     return hres;
 }
