@@ -71,6 +71,7 @@ static int strcmp_wa(LPCWSTR strw, const char *stra)
     }while(0)
 
 static IHTMLXMLHttpRequest *xhr = NULL;
+static BSTR content_type = NULL;
 
 DEFINE_EXPECT(xmlhttprequest_onreadystatechange_opened);
 DEFINE_EXPECT(xmlhttprequest_onreadystatechange_headers_received);
@@ -376,6 +377,11 @@ static void pump_msgs(BOOL *b)
 
 static const char EXPECT_RESPONSE_TEXT[] = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<a>TEST</a>\n";
 
+struct HEADER_TYPE {
+    const char *key;
+    const char *value;
+};
+
 static void create_xmlhttprequest(IHTMLDocument2 *doc)
 {
     IHTMLWindow2 *window;
@@ -412,6 +418,39 @@ static void create_xmlhttprequest(IHTMLDocument2 *doc)
     ok(xhr != NULL, "xhr == NULL\n");
 }
 
+static void test_header(const struct HEADER_TYPE expect[], int num)
+{
+    int i;
+    BSTR key, text, all_header;
+    HRESULT hres;
+    char all[4096], buf[512];
+
+    all_header = NULL;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &all_header);
+    ok(hres == S_OK, "getAllResponseHeader failed: %08x\n", hres);
+    ok(all_header != NULL, "all_header == NULL\n");
+
+    WideCharToMultiByte(CP_UTF8, 0, all_header, -1, all, sizeof(all), NULL, NULL);
+    SysFreeString(all_header);
+
+    for(i = 0; i < num; ++i) {
+        text = NULL;
+        key = a2bstr(expect[i].key);
+        hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, key, &text);
+        ok(hres == S_OK, "getResponseHeader failed, got %08x\n", hres);
+        ok(text != NULL, "text == NULL\n");
+        ok(!strcmp_wa(text, expect[i].value),
+            "Expect %s: %s, got %s\n", expect[i].key, expect[i].value, wine_dbgstr_w(text));
+        SysFreeString(key);
+        SysFreeString(text);
+
+        strcpy(buf, expect[i].key);
+        strcat(buf, ": ");
+        strcat(buf, expect[i].value);
+        ok(strstr(all, buf) != NULL, "AllResponseHeaders(%s) don't have expected substr(%s)\n", all, buf);
+    }
+}
+
 static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
 {
     VARIANT vbool, vempty, var;
@@ -419,6 +458,12 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     BSTR text;
     LONG val;
     HRESULT hres;
+    static const struct HEADER_TYPE expect_headers[] = {
+        {"Server", "Apache"},
+        {"Accept-Ranges", "bytes"},
+        {"Content-Length", "51"},
+        {"Content-Type", "application/xml"}
+    };
 
     create_xmlhttprequest(doc);
     if(!xhr)
@@ -457,11 +502,21 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     ok(val == 0, "Expect 0, got %d\n", val);
 
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, NULL);
-    todo_wine ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
+    ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
 
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(text == NULL, "Expect NULL, got %p\n", text);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(text == NULL, "Expect NULL, got %p\n", text);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
 
     method = a2bstr("GET");
     url = a2bstr(xml_url);
@@ -483,6 +538,16 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
         return;
     }
 
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
     ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
@@ -508,6 +573,14 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     CHECK_CALLED(xmlhttprequest_onreadystatechange_loading);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_done);
 
+    text = NULL;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == S_OK, "getResponseHeader failed, got %08x\n", hres);
+    ok(text != NULL, "text == NULL\n");
+    SysFreeString(text);
+
+    test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
+
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
     ok(hres == S_OK, "get_status failed: %08x\n", hres);
@@ -543,6 +616,10 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     BSTR text;
     LONG val;
     HRESULT hres;
+    static const struct HEADER_TYPE expect_headers[] = {
+        {"Content-Length", "51"},
+        {"Content-Type", "application/xml"}
+    };
 
     create_xmlhttprequest(doc);
     if(!xhr)
@@ -559,6 +636,29 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     ok(V_VT(&var) == VT_DISPATCH, "V_VT(onreadystatechange) = %d\n", V_VT(&var));
     ok(V_DISPATCH(&var) == (IDispatch*)&xmlhttprequest_onreadystatechange_obj, "unexpected onreadystatechange value\n");
 
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, NULL, &text);
+    ok(hres == E_INVALIDARG, "Expect E_INVALIDARG, got %08x\n", hres);
+
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, NULL);
+    ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
+
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, NULL, NULL);
+    ok(hres == E_POINTER || broken(hres == E_INVALIDARG), /* Vista and before */
+        "Expect E_POINTER, got %08x\n", hres);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, NULL);
+    ok(hres == E_POINTER, "Expect E_POINTER, got %08x\n", hres);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
     ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
@@ -566,8 +666,8 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     text = (BSTR)0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(text == NULL, "Expect NULL, got %p\n", text);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(text == NULL, "Expect NULL, got %p\n", text);
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_readyState(xhr, &val);
@@ -594,14 +694,24 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
         return;
     }
 
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
+    text = (BSTR)0xdeadbeef;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == E_FAIL, "got %08x\n", hres);
+    ok(text == NULL, "text = %p\n", text);
+
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
     ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
     ok(val == 0, "Expect 0, got %d\n", val);
 
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
-    todo_wine ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
-    todo_wine ok(text == NULL, "Expect NULL, got %p\n", text);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(text == NULL, "Expect NULL, got %p\n", text);
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_readyState(xhr, &val);
@@ -628,6 +738,21 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
         return;
     }
 
+    text = NULL;
+    hres = IHTMLXMLHttpRequest_getAllResponseHeaders(xhr, &text);
+    ok(hres == S_OK, "getAllResponseHeader failed, got %08x\n", hres);
+    ok(text != NULL, "text == NULL\n");
+    SysFreeString(text);
+
+    text = NULL;
+    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
+    ok(hres == S_OK, "getResponseHeader failed, got %08x\n", hres);
+    ok(text != NULL, "text == NULL\n");
+    ok(!strcmp_wa(text, "application/xml"), "text = %s\n", wine_dbgstr_w(text));
+    SysFreeString(text);
+
+    test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
+
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
     ok(hres == S_OK, "get_status failed: %08x\n", hres);
@@ -635,9 +760,9 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
 
     text = NULL;
     hres = IHTMLXMLHttpRequest_get_statusText(xhr, &text);
-    todo_wine ok(hres == S_OK, "get_statusText failed: %08x\n", hres);
-    todo_wine ok(text != NULL, "text == NULL\n");
-    todo_wine ok(!strcmp_wa(text, "OK"), "Expected \"OK\", got %s\n", wine_dbgstr_w(text));
+    ok(hres == S_OK, "get_statusText failed: %08x\n", hres);
+    ok(text != NULL, "text == NULL\n");
+    ok(!strcmp_wa(text, "OK"), "Expected \"OK\", got %s\n", wine_dbgstr_w(text));
     SysFreeString(text);
 
     val = 0xdeadbeef;
@@ -707,12 +832,14 @@ START_TEST(xmlhttprequest)
 
     CoInitialize(NULL);
 
+    content_type = a2bstr("Content-Type");
     doc = create_doc_from_url(start_url);
     if(doc) {
         test_sync_xhr(doc, xml_url);
         test_async_xhr(doc, xml_url);
         IHTMLDocument2_Release(doc);
     }
+    SysFreeString(content_type);
 
     CoUninitialize();
 }
