@@ -129,7 +129,7 @@ static void save_context( CONTEXT *context, const ucontext_t *sigcontext )
     context->Lr     = LR_sig(sigcontext);     /* Link register */
     context->Sp     = SP_sig(sigcontext);     /* Stack pointer */
     context->Pc     = PC_sig(sigcontext);     /* Program Counter */
-    context->PState = PSTATE_sig(sigcontext); /* Current State Register */
+    context->Cpsr   = PSTATE_sig(sigcontext); /* Current State Register */
 }
 
 
@@ -151,7 +151,7 @@ static void restore_context( const CONTEXT *context, ucontext_t *sigcontext )
     LR_sig(sigcontext)     = context->Lr;     /* Link register */
     SP_sig(sigcontext)     = context->Sp;     /* Stack pointer */
     PC_sig(sigcontext)     = context->Pc;     /* Program Counter */
-    PSTATE_sig(sigcontext) = context->PState; /* Current State Register */
+    PSTATE_sig(sigcontext) = context->Cpsr;   /* Current State Register */
 }
 
 
@@ -179,11 +179,52 @@ static inline void restore_fpu( CONTEXT *context, const ucontext_t *sigcontext )
 /***********************************************************************
  *		RtlCaptureContext (NTDLL.@)
  */
-void WINAPI RtlCaptureContext( CONTEXT *context )
-{
-    FIXME( "Not implemented on ARM64\n" );
-    memset( context, 0, sizeof(*context) );
-}
+/* FIXME: Use the Stack instead of the actual register values? */
+__ASM_STDCALL_FUNC( RtlCaptureContext, 8,
+                    "stp x0, x1, [sp, #-32]!\n\t"
+                    "mov w1, #0x400000\n\t"         /* CONTEXT_ARM64 */
+                    "add w1, w1, #0x3\n\t"          /* CONTEXT_FULL */
+                    "str w1, [x0]\n\t"              /* context->ContextFlags */ /* 32-bit, look at cpsr */
+                    "mrs x1, DAIF\n\t"
+                    "str w1, [x0, #0x4]\n\t"        /* context->Cpsr */
+                    "ldp x0, x1, [sp], #32\n\t"
+                    "str x0, [x0, #0x8]\n\t"        /* context->X0 */
+                    "str x1, [x0, #0x10]\n\t"       /* context->X1 */
+                    "str x2, [x0, #0x18]\n\t"       /* context->X2 */
+                    "str x3, [x0, #0x20]\n\t"       /* context->X3 */
+                    "str x4, [x0, #0x28]\n\t"       /* context->X4 */
+                    "str x5, [x0, #0x30]\n\t"       /* context->X5 */
+                    "str x6, [x0, #0x38]\n\t"       /* context->X6 */
+                    "str x7, [x0, #0x40]\n\t"       /* context->X7 */
+                    "str x8, [x0, #0x48]\n\t"       /* context->X8 */
+                    "str x9, [x0, #0x50]\n\t"       /* context->X9 */
+                    "str x10, [x0, #0x58]\n\t"      /* context->X10 */
+                    "str x11, [x0, #0x60]\n\t"      /* context->X11 */
+                    "str x12, [x0, #0x68]\n\t"      /* context->X12 */
+                    "str x13, [x0, #0x70]\n\t"      /* context->X13 */
+                    "str x14, [x0, #0x78]\n\t"      /* context->X14 */
+                    "str x15, [x0, #0x80]\n\t"      /* context->X15 */
+                    "str x16, [x0, #0x88]\n\t"      /* context->X16 */
+                    "str x17, [x0, #0x90]\n\t"      /* context->X17 */
+                    "str x18, [x0, #0x98]\n\t"      /* context->X18 */
+                    "str x19, [x0, #0xa0]\n\t"      /* context->X19 */
+                    "str x20, [x0, #0xa8]\n\t"      /* context->X20 */
+                    "str x21, [x0, #0xb0]\n\t"      /* context->X21 */
+                    "str x22, [x0, #0xb8]\n\t"      /* context->X22 */
+                    "str x23, [x0, #0xc0]\n\t"      /* context->X23 */
+                    "str x24, [x0, #0xc8]\n\t"      /* context->X24 */
+                    "str x25, [x0, #0xd0]\n\t"      /* context->X25 */
+                    "str x26, [x0, #0xd8]\n\t"      /* context->X26 */
+                    "str x27, [x0, #0xe0]\n\t"      /* context->X27 */
+                    "str x28, [x0, #0xe8]\n\t"      /* context->X28 */
+                    "str x29, [x0, #0xf0]\n\t"      /* context->Fp */
+                    "str x30, [x0, #0xf8]\n\t"      /* context->Lr */
+                    "mov x1, sp\n\t"
+                    "str x1, [x0, #0x100]\n\t"      /* context->Sp */
+                    "adr x1, 1f\n\t"
+                    "1: str x1, [x0, #0x108]\n\t"   /* context->Pc */
+                    "ret"
+                    )
 
 /***********************************************************************
  *           set_cpu_context
@@ -209,7 +250,7 @@ void copy_context( CONTEXT *to, const CONTEXT *from, DWORD flags )
         to->Lr      = from->Lr;
         to->Sp      = from->Sp;
         to->Pc      = from->Pc;
-        to->PState  = from->PState;
+        to->Cpsr    = from->Cpsr;
     }
     if (flags & CONTEXT_INTEGER)
     {
@@ -241,7 +282,7 @@ NTSTATUS context_to_server( context_t *to, const CONTEXT *from )
         to->integer.arm64_regs.x[30] = from->Lr;
         to->ctl.arm64_regs.sp     = from->Sp;
         to->ctl.arm64_regs.pc     = from->Pc;
-        to->ctl.arm64_regs.pstate = from->PState;
+        to->ctl.arm64_regs.pstate = from->Cpsr;
     }
     if (flags & CONTEXT_INTEGER)
     {
@@ -274,7 +315,7 @@ NTSTATUS context_from_server( CONTEXT *to, const context_t *from )
         to->Lr     = from->integer.arm64_regs.x[30];
         to->Sp     = from->ctl.arm64_regs.sp;
         to->Pc     = from->ctl.arm64_regs.pc;
-        to->PState = from->ctl.arm64_regs.pstate;
+        to->Cpsr   = from->ctl.arm64_regs.pstate;
     }
     if (from->flags & SERVER_CTX_INTEGER)
     {
@@ -758,6 +799,10 @@ void signal_init_thread( TEB *teb )
         pthread_key_create( &teb_key, NULL );
         init_done = TRUE;
     }
+
+    /* Win64/ARM applications expect the TEB pointer to be in the x18 platform register. */
+    __asm__ __volatile__( "mov x18, %0" : : "r" (teb) );
+
     pthread_setspecific( teb_key, teb );
 }
 
