@@ -48,7 +48,6 @@
 
 #define COBJMACROS
 #define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 
 #include "winerror.h"
 #include "windef.h"
@@ -2301,6 +2300,15 @@ HRESULT WINAPI OleLoadPictureEx( LPSTREAM lpstream, LONG lSize, BOOL fRunmode,
   return hr;
 }
 
+/***********************************************************************
+ * OleSavePictureFile (OLEAUT32.423)
+ */
+HRESULT WINAPI OleSavePictureFile(IDispatch *picture, BSTR filename)
+{
+  FIXME("(%p %s): stub\n", picture, debugstr_w(filename));
+  return CTL_E_FILENOTFOUND;
+}
+
 /***********************************************************************
  * OleLoadPicturePath (OLEAUT32.424)
  */
