@@ -29,6 +29,9 @@ WINE_DECLARE_DEBUG_CHANNEL(d3d_bytecode);
 #define WINED3D_SM4_INSTRUCTION_LENGTH_SHIFT    24
 #define WINED3D_SM4_INSTRUCTION_LENGTH_MASK     (0x1f << WINED3D_SM4_INSTRUCTION_LENGTH_SHIFT)
 
+#define WINED3D_SM4_RESOURCE_TYPE_SHIFT         11
+#define WINED3D_SM4_RESOURCE_TYPE_MASK          (0xf << WINED3D_SM4_RESOURCE_TYPE_SHIFT)
+
 #define WINED3D_SM4_PRIMITIVE_TYPE_SHIFT        11
 #define WINED3D_SM4_PRIMITIVE_TYPE_MASK         (0x7 << WINED3D_SM4_PRIMITIVE_TYPE_SHIFT)
 
@@ -108,6 +111,8 @@ enum wined3d_sm4_opcode
     WINED3D_SM4_OP_MOV                  = 0x36,
     WINED3D_SM4_OP_MOVC                 = 0x37,
     WINED3D_SM4_OP_MUL                  = 0x38,
+    WINED3D_SM4_OP_NE                   = 0x39,
+    WINED3D_SM4_OP_OR                   = 0x3c,
     WINED3D_SM4_OP_RET                  = 0x3e,
     WINED3D_SM4_OP_ROUND_NI             = 0x41,
     WINED3D_SM4_OP_RSQ                  = 0x44,
@@ -117,9 +122,11 @@ enum wined3d_sm4_opcode
     WINED3D_SM4_OP_SQRT                 = 0x4b,
     WINED3D_SM4_OP_SINCOS               = 0x4d,
     WINED3D_SM4_OP_UDIV                 = 0x4e,
+    WINED3D_SM4_OP_UGE                  = 0x50,
     WINED3D_SM4_OP_USHR                 = 0x55,
     WINED3D_SM4_OP_UTOF                 = 0x56,
     WINED3D_SM4_OP_XOR                  = 0x57,
+    WINED3D_SM4_OP_DCL_RESOURCE         = 0x58,
     WINED3D_SM4_OP_DCL_CONSTANT_BUFFER  = 0x59,
     WINED3D_SM4_OP_DCL_OUTPUT_TOPOLOGY  = 0x5c,
     WINED3D_SM4_OP_DCL_INPUT_PRIMITIVE  = 0x5d,
@@ -133,6 +140,7 @@ enum wined3d_sm4_register_type
     WINED3D_SM4_RT_OUTPUT       = 0x2,
     WINED3D_SM4_RT_IMMCONST     = 0x4,
     WINED3D_SM4_RT_SAMPLER      = 0x6,
+    WINED3D_SM4_RT_RESOURCE     = 0x7,
     WINED3D_SM4_RT_CONSTBUFFER  = 0x8,
     WINED3D_SM4_RT_PRIMID       = 0xb,
     WINED3D_SM4_RT_NULL         = 0xd,
@@ -166,6 +174,28 @@ enum wined3d_sm4_immconst_type
     WINED3D_SM4_IMMCONST_VEC4   = 0x2,
 };
 
+enum wined3d_sm4_resource_type
+{
+    WINED3D_SM4_RESOURCE_BUFFER             = 0x1,
+    WINED3D_SM4_RESOURCE_TEXTURE_1D         = 0x2,
+    WINED3D_SM4_RESOURCE_TEXTURE_2D         = 0x3,
+    WINED3D_SM4_RESOURCE_TEXTURE_2DMS       = 0x4,
+    WINED3D_SM4_RESOURCE_TEXTURE_3D         = 0x5,
+    WINED3D_SM4_RESOURCE_TEXTURE_CUBE       = 0x6,
+    WINED3D_SM4_RESOURCE_TEXTURE_1DARRAY    = 0x7,
+    WINED3D_SM4_RESOURCE_TEXTURE_2DARRAY    = 0x8,
+    WINED3D_SM4_RESOURCE_TEXTURE_2DMSARRAY  = 0x9,
+};
+
+enum wined3d_sm4_data_type
+{
+    WINED3D_SM4_DATA_UNORM  = 0x1,
+    WINED3D_SM4_DATA_SNORM  = 0x2,
+    WINED3D_SM4_DATA_INT    = 0x3,
+    WINED3D_SM4_DATA_UINT   = 0x4,
+    WINED3D_SM4_DATA_FLOAT  = 0x5,
+};
+
 struct wined3d_shader_src_param_entry
 {
     struct list entry;
@@ -176,7 +206,12 @@ struct wined3d_sm4_data
 {
     struct wined3d_shader_version shader_version;
     const DWORD *end;
-    const struct wined3d_shader_signature *output_signature;
+
+    struct
+    {
+        enum wined3d_shader_register_type register_type;
+        UINT register_idx;
+    } output_map[MAX_REG_OUTPUT];
 
     struct wined3d_shader_src_param src_param[5];
     struct wined3d_shader_dst_param dst_param[2];
@@ -245,6 +280,8 @@ static const struct wined3d_sm4_opcode_info opcode_table[] =
     {WINED3D_SM4_OP_MOV,                    WINED3DSIH_MOV,                 "F",    "F"},
     {WINED3D_SM4_OP_MOVC,                   WINED3DSIH_MOVC,                "F",    "UFF"},
     {WINED3D_SM4_OP_MUL,                    WINED3DSIH_MUL,                 "F",    "FF"},
+    {WINED3D_SM4_OP_NE,                     WINED3DSIH_NE,                  "U",    "FF"},
+    {WINED3D_SM4_OP_OR,                     WINED3DSIH_OR,                  "U",    "UU"},
     {WINED3D_SM4_OP_RET,                    WINED3DSIH_RET,                 "",     ""},
     {WINED3D_SM4_OP_ROUND_NI,               WINED3DSIH_ROUND_NI,            "F",    "F"},
     {WINED3D_SM4_OP_RSQ,                    WINED3DSIH_RSQ,                 "F",    "F"},
@@ -254,9 +291,11 @@ static const struct wined3d_sm4_opcode_info opcode_table[] =
     {WINED3D_SM4_OP_SQRT,                   WINED3DSIH_SQRT,                "F",    "F"},
     {WINED3D_SM4_OP_SINCOS,                 WINED3DSIH_SINCOS,              "FF",   "F"},
     {WINED3D_SM4_OP_UDIV,                   WINED3DSIH_UDIV,                "UU",   "UU"},
+    {WINED3D_SM4_OP_UGE,                    WINED3DSIH_UGE,                 "U",    "UU"},
     {WINED3D_SM4_OP_USHR,                   WINED3DSIH_USHR,                "U",    "UU"},
     {WINED3D_SM4_OP_UTOF,                   WINED3DSIH_UTOF,                "F",    "U"},
     {WINED3D_SM4_OP_XOR,                    WINED3DSIH_XOR,                 "U",    "UU"},
+    {WINED3D_SM4_OP_DCL_RESOURCE,           WINED3DSIH_DCL,                 "R",    ""},
     {WINED3D_SM4_OP_DCL_CONSTANT_BUFFER,    WINED3DSIH_DCL_CONSTANT_BUFFER, "",     ""},
     {WINED3D_SM4_OP_DCL_OUTPUT_TOPOLOGY,    WINED3DSIH_DCL_OUTPUT_TOPOLOGY, "",     ""},
     {WINED3D_SM4_OP_DCL_INPUT_PRIMITIVE,    WINED3DSIH_DCL_INPUT_PRIMITIVE, "",     ""},
@@ -316,6 +355,30 @@ static const struct sysval_map sysval_map[] =
     {WINED3D_SV_TARGET7,    WINED3DSPR_COLOROUT,    7},
 };
 
+static const enum wined3d_shader_resource_type resource_type_table[] =
+{
+    /* 0 */                                         WINED3D_SHADER_RESOURCE_NONE,
+    /* WINED3D_SM4_RESOURCE_BUFFER */               WINED3D_SHADER_RESOURCE_BUFFER,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_1D */           WINED3D_SHADER_RESOURCE_TEXTURE_1D,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_2D */           WINED3D_SHADER_RESOURCE_TEXTURE_2D,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_2DMS */         WINED3D_SHADER_RESOURCE_TEXTURE_2DMS,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_3D */           WINED3D_SHADER_RESOURCE_TEXTURE_3D,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_CUBE */         WINED3D_SHADER_RESOURCE_TEXTURE_CUBE,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_1DARRAY */      WINED3D_SHADER_RESOURCE_TEXTURE_1DARRAY,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_2DARRAY */      WINED3D_SHADER_RESOURCE_TEXTURE_2DARRAY,
+    /* WINED3D_SM4_RESOURCE_TEXTURE_2DMSARRAY */    WINED3D_SHADER_RESOURCE_TEXTURE_2DMSARRAY,
+};
+
+static const enum wined3d_data_type data_type_table[] =
+{
+    /* 0 */                         WINED3D_DATA_FLOAT,
+    /* WINED3D_SM4_DATA_UNORM */    WINED3D_DATA_UNORM,
+    /* WINED3D_SM4_DATA_SNORM */    WINED3D_DATA_SNORM,
+    /* WINED3D_SM4_DATA_INT */      WINED3D_DATA_INT,
+    /* WINED3D_SM4_DATA_UINT */     WINED3D_DATA_UINT,
+    /* WINED3D_SM4_DATA_FLOAT */    WINED3D_DATA_FLOAT,
+};
+
 static BOOL shader_sm4_read_src_param(struct wined3d_sm4_data *priv, const DWORD **ptr,
         enum wined3d_data_type data_type, struct wined3d_shader_src_param *src_param);
 
@@ -331,20 +394,6 @@ static const struct wined3d_sm4_opcode_info *get_opcode_info(enum wined3d_sm4_op
     return NULL;
 }
 
-static void map_sysval(enum wined3d_sysval_semantic sysval, struct wined3d_shader_register *reg)
-{
-    unsigned int i;
-
-    for (i = 0; i < sizeof(sysval_map) / sizeof(*sysval_map); ++i)
-    {
-        if (sysval == sysval_map[i].sysval)
-        {
-            reg->type = sysval_map[i].register_type;
-            reg->idx[0].offset = sysval_map[i].register_idx;
-        }
-    }
-}
-
 static void map_register(const struct wined3d_sm4_data *priv, struct wined3d_shader_register *reg)
 {
     switch (priv->shader_version.type)
@@ -352,23 +401,16 @@ static void map_register(const struct wined3d_sm4_data *priv, struct wined3d_sha
         case WINED3D_SHADER_TYPE_PIXEL:
             if (reg->type == WINED3DSPR_OUTPUT)
             {
-                unsigned int i;
-                const struct wined3d_shader_signature *s = priv->output_signature;
+                unsigned int reg_idx = reg->idx[0].offset;
 
-                if (!s)
+                if (reg_idx >= ARRAY_SIZE(priv->output_map))
                 {
-                    ERR("Shader has no output signature, unable to map register.\n");
+                    ERR("Invalid output index %u.\n", reg_idx);
                     break;
                 }
 
-                for (i = 0; i < s->element_count; ++i)
-                {
-                    if (s->elements[i].register_idx == reg->idx[0].offset)
-                    {
-                        map_sysval(s->elements[i].sysval_semantic, reg);
-                        break;
-                    }
-                }
+                reg->type = priv->output_map[reg_idx].register_type;
+                reg->idx[0].offset = priv->output_map[reg_idx].register_idx;
             }
             break;
 
@@ -399,14 +441,37 @@ static enum wined3d_data_type map_data_type(char t)
 
 static void *shader_sm4_init(const DWORD *byte_code, const struct wined3d_shader_signature *output_signature)
 {
-    struct wined3d_sm4_data *priv = HeapAlloc(GetProcessHeap(), 0, sizeof(*priv));
-    if (!priv)
+    struct wined3d_sm4_data *priv;
+    unsigned int i, j;
+
+    if (!(priv = HeapAlloc(GetProcessHeap(), 0, sizeof(*priv))))
     {
         ERR("Failed to allocate private data\n");
         return NULL;
     }
 
-    priv->output_signature = output_signature;
+    memset(priv->output_map, 0xff, sizeof(priv->output_map));
+    for (i = 0; i < output_signature->element_count; ++i)
+    {
+        struct wined3d_shader_signature_element *e = &output_signature->elements[i];
+
+        if (e->register_idx >= ARRAY_SIZE(priv->output_map))
+        {
+            WARN("Invalid output index %u.\n", e->register_idx);
+            continue;
+        }
+
+        for (j = 0; j < ARRAY_SIZE(sysval_map); ++j)
+        {
+            if (e->sysval_semantic == sysval_map[j].sysval)
+            {
+                priv->output_map[e->register_idx].register_type = sysval_map[j].register_type;
+                priv->output_map[e->register_idx].register_idx = sysval_map[j].register_idx;
+                break;
+            }
+        }
+    }
+
     list_init(&priv->src_free);
     list_init(&priv->src);
 
@@ -734,7 +799,40 @@ static void shader_sm4_read_instruction(void *data, const DWORD **ptr, struct wi
         FIXME("Skipping modifier 0x%08x.\n", modifier);
     }
 
-    if (opcode == WINED3D_SM4_OP_DCL_CONSTANT_BUFFER)
+    if (opcode == WINED3D_SM4_OP_DCL_RESOURCE)
+    {
+        enum wined3d_sm4_resource_type resource_type;
+        enum wined3d_sm4_data_type data_type;
+        DWORD components;
+
+        resource_type = (opcode_token & WINED3D_SM4_RESOURCE_TYPE_MASK) >> WINED3D_SM4_RESOURCE_TYPE_SHIFT;
+        if (!resource_type || (resource_type >= ARRAY_SIZE(resource_type_table)))
+        {
+            FIXME("Unhandled resource type %#x.\n", resource_type);
+            ins->declaration.semantic.resource_type = WINED3D_SHADER_RESOURCE_NONE;
+        }
+        else
+        {
+            ins->declaration.semantic.resource_type = resource_type_table[resource_type];
+        }
+        shader_sm4_read_dst_param(priv, &p, WINED3D_DATA_RESOURCE, &ins->declaration.semantic.reg);
+
+        components = *p++;
+        if ((components & 0xfff0) != (components & 0xf) * 0x1110)
+            FIXME("Components (%#x) have different data types.\n", components);
+        data_type = components & 0xf;
+
+        if (!data_type || (data_type >= ARRAY_SIZE(data_type_table)))
+        {
+            FIXME("Unhandled data type %#x.\n", data_type);
+            ins->declaration.semantic.resource_data_type = WINED3D_DATA_FLOAT;
+        }
+        else
+        {
+            ins->declaration.semantic.resource_data_type = data_type_table[data_type];
+        }
+    }
+    else if (opcode == WINED3D_SM4_OP_DCL_CONSTANT_BUFFER)
     {
         shader_sm4_read_src_param(priv, &p, WINED3D_DATA_FLOAT, &ins->declaration.src);
         if (opcode_token & WINED3D_SM4_INDEX_TYPE_MASK)
