@@ -72,12 +72,12 @@ static void get_cocoa_window_features(struct macdrv_win_data *data,
             if (ex_style & WS_EX_TOOLWINDOW) wf->utility = TRUE;
         }
     }
-    if (ex_style & WS_EX_DLGMODALFRAME) wf->shadow = TRUE;
-    else if (style & WS_THICKFRAME)
+    if (style & WS_THICKFRAME)
     {
         wf->shadow = TRUE;
         if (!data->shaped) wf->resizable = TRUE;
     }
+    else if (ex_style & WS_EX_DLGMODALFRAME) wf->shadow = TRUE;
     else if ((style & (WS_DLGFRAME|WS_BORDER)) == WS_DLGFRAME) wf->shadow = TRUE;
 }
 
@@ -1274,6 +1274,8 @@ void CDECL macdrv_DestroyWindow(HWND hwnd)
 
     if (!(data = get_win_data(hwnd))) return;
 
+    if (hwnd == GetCapture()) macdrv_SetCapture(0, 0);
+
     if (data->gl_view) macdrv_dispose_view(data->gl_view);
     destroy_cocoa_window(data);
 
@@ -2139,8 +2141,28 @@ void macdrv_window_resize_ended(HWND hwnd)
  * Handler for WINDOW_RESTORE_REQUESTED events.  This is specifically
  * for restoring from maximized, not from minimized.
  */
-void macdrv_window_restore_requested(HWND hwnd)
+void macdrv_window_restore_requested(HWND hwnd, const macdrv_event *event)
 {
+    if (event->window_restore_requested.keep_frame && hwnd)
+    {
+        DWORD style = GetWindowLongW(hwnd, GWL_STYLE);
+        struct macdrv_win_data *data;
+
+        if ((style & WS_MAXIMIZE) && (style & WS_VISIBLE) && (data = get_win_data(hwnd)))
+        {
+            RECT rect;
+            HWND parent = GetAncestor(hwnd, GA_PARENT);
+
+            rect = rect_from_cgrect(event->window_restore_requested.frame);
+            macdrv_mac_to_window_rect(data, &rect);
+            MapWindowPoints(0, parent, (POINT *)&rect, 2);
+
+            release_win_data(data);
+
+            SetInternalWindowPos(hwnd, SW_SHOW, &rect, NULL);
+        }
+    }
+
     perform_window_command(hwnd, WS_MAXIMIZE, 0, SC_RESTORE, HTMAXBUTTON);
 }
 
@@ -2173,14 +2195,18 @@ void macdrv_window_drag_begin(HWND hwnd)
 
     while (GetMessageW(&msg, 0, 0, 0))
     {
+        if (msg.message == WM_EXITSIZEMOVE)
+        {
+            SendMessageW(hwnd, WM_EXITSIZEMOVE, 0, 0);
+            break;
+        }
+
         if (!CallMsgFilterW(&msg, MSGF_SIZE) && msg.message != WM_KEYDOWN &&
             msg.message != WM_MOUSEMOVE && msg.message != WM_LBUTTONDOWN && msg.message != WM_LBUTTONUP)
         {
             TranslateMessage(&msg);
             DispatchMessageW(&msg);
         }
-
-        if (msg.message == WM_EXITSIZEMOVE) break;
     }
 
     TRACE("done\n");
