@@ -363,6 +363,7 @@ static void create_and_write_file(LPCSTR filename, void *data, DWORD len)
 
 static void test_urlcacheA(void)
 {
+    static char long_url[300] = "http://www.winehq.org/";
     static char ok_header[] = "HTTP/1.0 200 OK\r\n\r\n";
     BOOL ret;
     HANDLE hFile;
@@ -372,6 +373,7 @@ static void test_urlcacheA(void)
     DWORD cbCacheEntryInfo;
     static const FILETIME filetime_zero;
     FILETIME now;
+    int len;
 
     ret = CreateUrlCacheEntryA(test_url, 0, "html", filenameA, 0);
     ok(ret, "CreateUrlCacheEntry failed with error %d\n", GetLastError());
@@ -807,6 +809,30 @@ static void test_urlcacheA(void)
         ret = pDeleteUrlCacheEntryA(test_hash_collisions2);
         ok(ret, "DeleteUrlCacheEntry failed: %d\n", GetLastError());
     }
+
+    len = strlen(long_url);
+    memset(long_url+len, 'a', sizeof(long_url)-len);
+    long_url[sizeof(long_url)-1] = 0;
+    ret = CreateUrlCacheEntryA(long_url, 0, NULL, filenameA, 0);
+    ok(ret, "CreateUrlCacheEntry failed with error %d\n", GetLastError());
+    check_file_exists(filenameA);
+    DeleteFileA(filenameA);
+
+    ret = CreateUrlCacheEntryA(long_url, 0, "extension", filenameA, 0);
+    ok(ret, "CreateUrlCacheEntry failed with error %d\n", GetLastError());
+    check_file_exists(filenameA);
+    DeleteFileA(filenameA);
+
+    long_url[250] = 0;
+    ret = CreateUrlCacheEntryA(long_url, 0, NULL, filenameA, 0);
+    ok(ret, "CreateUrlCacheEntry failed with error %d\n", GetLastError());
+    check_file_exists(filenameA);
+    DeleteFileA(filenameA);
+
+    ret = CreateUrlCacheEntryA(long_url, 0, "extension", filenameA, 0);
+    ok(ret, "CreateUrlCacheEntry failed with error %d\n", GetLastError());
+    check_file_exists(filenameA);
+    DeleteFileA(filenameA);
 }
 
 static void test_urlcacheW(void)
