@@ -184,11 +184,14 @@ extern "C" {
 #endif /* USE_WS_PREFIX */
 
 /* Constants for WSAIoctl() */
-#define WSA_FLAG_OVERLAPPED        0x01
-#define WSA_FLAG_MULTIPOINT_C_ROOT 0x02
-#define WSA_FLAG_MULTIPOINT_C_LEAF 0x04
-#define WSA_FLAG_MULTIPOINT_D_ROOT 0x08
-#define WSA_FLAG_MULTIPOINT_D_LEAF 0x10
+#define WSA_FLAG_OVERLAPPED             0x0001
+#define WSA_FLAG_MULTIPOINT_C_ROOT      0x0002
+#define WSA_FLAG_MULTIPOINT_C_LEAF      0x0004
+#define WSA_FLAG_MULTIPOINT_D_ROOT      0x0008
+#define WSA_FLAG_MULTIPOINT_D_LEAF      0x0010
+#define WSA_FLAG_ACCESS_SYSTEM_SECURITY 0x0040
+#define WSA_FLAG_NO_HANDLE_INHERIT      0x0080
+#define WSA_FLAG_REGISTERED_IO          0x0100
 
 /* Constants for WSAJoinLeaf() */
 #define JL_SENDER_ONLY    0x01
