@@ -21,7 +21,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
 
 #include "wine/unicode.h"
 #include "wine/debug.h"
@@ -639,6 +638,7 @@ static int copy_path_string(LPWSTR target, LPWSTR source)
         while (*source && *source != ',') target[i++] = *source++;
         target[i] = 0;
     }
+    PathRemoveBackslashW(target);
     return i;
 }
 
