@@ -22,7 +22,6 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
-#define NONAMELESSUNION
 #include <windows.h>
 #include <commdlg.h>
 #include <wine/debug.h>
@@ -327,7 +326,7 @@ static void on_add_app_click(HWND dialog)
   static const WCHAR pathC[] = { 'c',':','\\',0 };
 
   OPENFILENAMEW ofn = { sizeof(OPENFILENAMEW),
-		       0, /*hInst*/0, 0, NULL, 0, 0, NULL,
+		       dialog, /*hInst*/0, 0, NULL, 0, 0, NULL,
 		       0, NULL, 0, pathC, 0,
 		       OFN_SHOWHELP | OFN_HIDEREADONLY | OFN_ENABLESIZING,
                        0, 0, NULL, 0, NULL };
@@ -377,26 +376,24 @@ static void on_remove_app_click(HWND dialog)
 {
     HWND listview = GetDlgItem(dialog, IDC_APP_LISTVIEW);
     int selection = get_listview_selection(listview);
-    char *section = keypath(""); /* AppDefaults\\whatever.exe\\ */
     LVITEMW item;
 
     item.iItem = selection;
     item.iSubItem = 0;
     item.mask = LVIF_PARAM;
 
-    WINE_TRACE("selection=%d, section=%s\n", selection, section);
-    
+    WINE_TRACE("selection=%d\n", selection);
+
     assert( selection != 0 ); /* user cannot click this button when "default settings" is selected  */
 
-    section[strlen(section)] = '\0'; /* remove last backslash  */
-    set_reg_key(config_key, section, NULL, NULL); /* delete the section  */
+    set_reg_key(config_key, keypath(""), NULL, NULL); /* delete the section  */
     SendMessageW(listview, LVM_GETITEMW, 0, (LPARAM) &item);
     HeapFree (GetProcessHeap(), 0, (void*)item.lParam);
     SendMessageW(listview, LVM_DELETEITEM, selection, 0);
     item.mask = LVIF_STATE;
     item.state = LVIS_SELECTED | LVIS_FOCUSED;
     item.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
-    SendMessageW(listview, LVM_SETITEMSTATE, -1, (LPARAM)&item);
+    SendMessageW(listview, LVM_SETITEMSTATE, 0, (LPARAM)&item);
 
     SetFocus(listview);
     SendMessageW(GetParent(dialog), PSM_CHANGED, (WPARAM) dialog, 0);
