@@ -577,3 +577,21 @@ DECL_HANDLER(unlink_object)
         release_object( obj );
     }
 }
+
+/* query object type name information */
+DECL_HANDLER(get_object_type)
+{
+    struct object *obj;
+    struct object_type *type;
+    const WCHAR *name;
+
+    if (!(obj = get_handle_obj( current->process, req->handle, 0, NULL ))) return;
+
+    if ((type = obj->ops->get_type( obj )))
+    {
+        if ((name = get_object_name( &type->obj, &reply->total )))
+            set_reply_data( name, min( reply->total, get_reply_max_size() ) );
+        release_object( type );
+    }
+    release_object( obj );
+}
