@@ -138,6 +138,13 @@ extern obj_handle_t open_mapping_file( struct process *process, struct mapping *
 extern struct mapping *grab_mapping_unless_removable( struct mapping *mapping );
 extern int get_page_size(void);
 
+/* device functions */
+
+extern void create_named_pipe_device( struct directory *root, const struct unicode_str *name );
+extern void create_mailslot_device( struct directory *root, const struct unicode_str *name );
+extern struct device *create_unix_device( struct directory *root, const struct unicode_str *name,
+                                          const char *unix_path );
+
 /* change notification functions */
 
 extern void do_change_notify( int unix_fd );
