@@ -869,7 +869,7 @@ const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec )
 }
 
 /* parse a cpu name and return the corresponding value */
-enum target_cpu get_cpu_from_name( const char *name )
+int get_cpu_from_name( const char *name )
 {
     unsigned int i;
 
@@ -931,11 +931,13 @@ unsigned int get_page_size(void)
 {
     switch(target_cpu)
     {
-    case CPU_x86:     return 4096;
-    case CPU_x86_64:  return 4096;
-    case CPU_POWERPC: return 4096;
-    case CPU_ARM:     return 4096;
-    case CPU_ARM64:   return 4096;
+    case CPU_x86:
+    case CPU_x86_64:
+    case CPU_POWERPC:
+    case CPU_ARM:
+        return 0x1000;
+    case CPU_ARM64:
+        return 0x10000;
     }
     /* unreached */
     assert(0);
